#!/usr/bin/env python

from time import sleep
from collections import deque
import numpy as np
from zoetrope import aniplot as plt
from scipy.stats.stats import pearsonr
from bstrd import BS, bsstream
plt.blit = False
plt.style.use('ggplot')

def pumpedshots(events, *arrays):
    laser = events[:, 18]
    darkShot = events[:, 21]
    pumped_shots = np.logical_and.reduce((laser, np.logical_not(darkShot)))
    return [a[pumped_shots] for a in arrays]

chname_tt = "SARES11-SPEC125-M1.edge_position"
chname_stage = "SLAAR11-LMOT-M452:ENC_1_BS" 
chname_events = "SAR-CVME-TIFALL4:EvtSet"
length = 500

# create channel
ch_tt  = BS(chname_tt)
ch_stage = BS(chname_stage)
ch_events = BS(chname_events)

n = 200
tt = np.empty(n)
stage = np.empty(n)
evts = np.empty((n, 256))

# create the empty plot
pd = plt.plot([0])

# some plot settings
plt.fig.set_figheight(6)
plt.fig.set_figwidth(7)

for counter, data in zip(plt.show(), bsstream):
    print(counter)

    for i in range(n):
        tt[i] = ch_tt.get()
        stage[i] = ch_stage.get()
        evts[i] = ch_events.get()
        next(bsstream) # this gets the next set of data

    pumpstage, pumptt = pumpedshots(evts, stage, tt)
    pearscoeff1, _ = pearsonr(stage, tt)

    #xs = np.arange(len(pp_sigs))
    plt.tight_layout()
    plt.clf()
    plt.scatter(pumpstage, pumptt)
    plt.title("{}".format(round(pearscoeff1, 4)))
    plt.xlabel('m452')
    plt.ylabel('arrival time')
    plt.show()
    
bsstream.close()
